@@ -281,40 +281,26 @@ GSList *nicklist_getnicks(CHANNEL_REC *channel)
 	return list;
 }
 
-typedef struct {
-        CHANNEL_REC *channel;
-	const char *nick;
-	GSList *list;
-} NICKLIST_GET_SAME_REC;
-
-static void get_nicks_same_hash(gpointer key, NICK_REC *nick,
-				NICKLIST_GET_SAME_REC *rec)
-{
-	while (nick != NULL) {
-		if (g_strcasecmp(nick->nick, rec->nick) == 0) {
-			rec->list = g_slist_append(rec->list, rec->channel);
-			rec->list = g_slist_append(rec->list, nick);
-		}
-
-		nick = nick->next;
-	}
-}
-
 GSList *nicklist_get_same(SERVER_REC *server, const char *nick)
 {
-	NICKLIST_GET_SAME_REC rec;
 	GSList *tmp;
+	GSList *list = NULL;
 
 	g_return_val_if_fail(IS_SERVER(server), NULL);
 
-	rec.nick = nick;
-	rec.list = NULL;
 	for (tmp = server->channels; tmp != NULL; tmp = tmp->next) {
-		rec.channel = tmp->data;
-		g_hash_table_foreach(rec.channel->nicks,
-				     (GHFunc) get_nicks_same_hash, &rec);
+		NICK_REC *nick_rec;
+		CHANNEL_REC *channel = tmp->data;
+
+		for (nick_rec = g_hash_table_lookup(channel->nicks, nick);
+		     nick_rec != NULL;
+		     nick_rec = nick_rec->next) {
+			list = g_slist_append(list, channel);
+			list = g_slist_append(list, nick_rec);
+		}
 	}
-	return rec.list;
+
+	return list;
 }
 
 typedef struct {
@@ -365,7 +351,7 @@ int nicklist_compare(NICK_REC *p1, NICK_REC *p2, const char *nick_prefix)
 	if (p2 == NULL) return 1;
 
 	if (p1->prefixes[0] == p2->prefixes[0])
-		return g_strcasecmp(p1->nick, p2->nick);
+		return g_ascii_strcasecmp(p1->nick, p2->nick);
 
 	if (!p1->prefixes[0])
 		return 1;
@@ -384,7 +370,7 @@ int nicklist_compare(NICK_REC *p1, NICK_REC *p2, const char *nick_prefix)
 	}
 
 	/* we should never have gotten here... */
-	return g_strcasecmp(p1->nick, p2->nick);
+	return g_ascii_strcasecmp(p1->nick, p2->nick);
 }
 
 static void nicklist_update_flags_list(SERVER_REC *server, int gone,
@@ -516,7 +502,8 @@ int nick_match_msg(CHANNEL_REC *channel, const char *msg, const char *nick)
 
 	/* first check for identical match */
 	len = strlen(nick);
-	if (g_strncasecmp(msg, nick, len) == 0 && !isalnumhigh((int) msg[len]))
+	if (g_ascii_strncasecmp(msg, nick, len) == 0 &&
+	    !isalnumhigh((int) msg[len]))
 		return TRUE;
 
 	orignick = nick;
