@@ -169,6 +169,8 @@ static void server_setup_fill_server(SERVER_CONNECT_REC *conn,
 		conn->ssl_cert = g_strdup(sserver->ssl_cert);
 	if (conn->ssl_pkey == NULL && sserver->ssl_pkey != NULL && sserver->ssl_pkey[0] != '\0')
 		conn->ssl_pkey = g_strdup(sserver->ssl_pkey);
+	if (conn->ssl_pass == NULL && sserver->ssl_pass != NULL && sserver->ssl_pass[0] != '\0')
+		conn->ssl_pass = g_strdup(sserver->ssl_pass);
 	conn->ssl_verify = sserver->ssl_verify;
 	if (conn->ssl_cafile == NULL && sserver->ssl_cafile != NULL && sserver->ssl_cafile[0] != '\0')
 		conn->ssl_cafile = g_strdup(sserver->ssl_cafile);
@@ -280,7 +282,7 @@ create_chatnet_conn(const char *dest, int port,
 		SERVER_SETUP_REC *rec = tmp->data;
 
 		if (rec->chatnet == NULL ||
-		    g_strcasecmp(rec->chatnet, dest) != 0)
+		    g_ascii_strcasecmp(rec->chatnet, dest) != 0)
 			continue;
 
 		if (!rec->last_failed) {
@@ -340,9 +342,9 @@ SERVER_SETUP_REC *server_setup_find(const char *address, int port,
 	for (tmp = setupservers; tmp != NULL; tmp = tmp->next) {
 		SERVER_SETUP_REC *rec = tmp->data;
 
-		if (g_strcasecmp(rec->address, address) == 0 &&
+		if (g_ascii_strcasecmp(rec->address, address) == 0 &&
 		    (chatnet == NULL || rec->chatnet == NULL ||
-		     g_strcasecmp(rec->chatnet, chatnet) == 0)) {
+		     g_ascii_strcasecmp(rec->chatnet, chatnet) == 0)) {
 			server = rec;
 			if (rec->port == port)
 				break;
@@ -352,15 +354,6 @@ SERVER_SETUP_REC *server_setup_find(const char *address, int port,
 	return server;
 }
 
-/* Find matching server from setup. Ports must match or NULL is returned. */
-SERVER_SETUP_REC *server_setup_find_port(const char *address, int port)
-{
-	SERVER_SETUP_REC *rec;
-
-	rec = server_setup_find(address, port, NULL);
-	return rec == NULL || rec->port != port ? NULL : rec;
-}
-
 static SERVER_SETUP_REC *server_setup_read(CONFIG_NODE *node)
 {
 	SERVER_SETUP_REC *rec;
@@ -375,14 +368,13 @@ static SERVER_SETUP_REC *server_setup_read(CONFIG_NODE *node)
 		return NULL;
 
 	port = config_node_get_int(node, "port", 0);
-	if (server_setup_find_port(server, port) != NULL) {
-		/* already exists - don't let it get there twice or
-		   server reconnects will screw up! */
+	chatnet = config_node_get_str(node, "chatnet", NULL);
+
+	if (server_setup_find(server, port, chatnet) != NULL) {
 		return NULL;
 	}
 
 	rec = NULL;
-	chatnet = config_node_get_str(node, "chatnet", NULL);
 
 	chatnetrec = chatnet == NULL ? NULL : chatnet_find(chatnet);
 	if (chatnetrec == NULL && chatnet != NULL) {
@@ -406,6 +398,7 @@ static SERVER_SETUP_REC *server_setup_read(CONFIG_NODE *node)
 	rec->use_ssl = config_node_get_bool(node, "use_ssl", FALSE);
 	rec->ssl_cert = g_strdup(config_node_get_str(node, "ssl_cert", NULL));
 	rec->ssl_pkey = g_strdup(config_node_get_str(node, "ssl_pkey", NULL));
+	rec->ssl_pass = g_strdup(config_node_get_str(node, "ssl_pass", NULL));
 	rec->ssl_verify = config_node_get_bool(node, "ssl_verify", FALSE);
 	rec->ssl_cafile = g_strdup(config_node_get_str(node, "ssl_cafile", NULL));
 	rec->ssl_capath = g_strdup(config_node_get_str(node, "ssl_capath", NULL));
@@ -445,6 +438,7 @@ static void server_setup_save(SERVER_SETUP_REC *rec)
 	iconfig_node_set_bool(node, "use_ssl", rec->use_ssl);
 	iconfig_node_set_str(node, "ssl_cert", rec->ssl_cert);
 	iconfig_node_set_str(node, "ssl_pkey", rec->ssl_pkey);
+	iconfig_node_set_str(node, "ssl_pass", rec->ssl_pass);
 	iconfig_node_set_bool(node, "ssl_verify", rec->ssl_verify);
 	iconfig_node_set_str(node, "ssl_cafile", rec->ssl_cafile);
 	iconfig_node_set_str(node, "ssl_capath", rec->ssl_capath);
@@ -486,6 +480,7 @@ static void server_setup_destroy(SERVER_SETUP_REC *rec)
 	g_free_not_null(rec->password);
 	g_free_not_null(rec->ssl_cert);
 	g_free_not_null(rec->ssl_pkey);
+	g_free_not_null(rec->ssl_pass);
 	g_free_not_null(rec->ssl_cafile);
 	g_free_not_null(rec->ssl_capath);
 	g_free(rec->address);
