@@ -90,9 +90,9 @@ void dcc_init_rec(DCC_REC *dcc, IRC_SERVER_REC *server, CHAT_DCC_REC *chat,
 
 	dcc->servertag = server != NULL ? g_strdup(server->tag) :
 		(chat == NULL ? NULL : g_strdup(chat->servertag));
-	
+
 	dcc->pasv_id = -1; /* Not a passive DCC */
-	
+
 	dcc_conns = g_slist_append(dcc_conns, dcc);
 	signal_emit("dcc created", 1, dcc);
 }
@@ -148,7 +148,7 @@ DCC_REC *dcc_find_request(int type, const char *nick, const char *arg)
 		DCC_REC *dcc = tmp->data;
 
 		if (dcc->type == type && !dcc_is_connected(dcc) &&
-		    g_strcasecmp(dcc->nick, nick) == 0 &&
+		    g_ascii_strcasecmp(dcc->nick, nick) == 0 &&
 		    (arg == NULL || strcmp(dcc->arg, arg) == 0))
 			return dcc;
 	}
@@ -287,7 +287,7 @@ static void sig_connected(IRC_SERVER_REC *server)
 		DCC_REC *dcc = tmp->data;
 
 		if (dcc->server == NULL && dcc->servertag != NULL &&
-		    g_strcasecmp(dcc->servertag, server->tag) == 0) {
+		    g_ascii_strcasecmp(dcc->servertag, server->tag) == 0) {
 			dcc->server = server;
 			g_free(dcc->mynick);
 			dcc->mynick = g_strdup(server->nick);
@@ -454,8 +454,8 @@ static int dcc_timeout_func(void)
 			/* Timed out - don't send DCC REJECT CTCP so CTCP
 			   flooders won't affect us and it really doesn't
 			   matter that much anyway if the other side doen't
-			   get it.. 
-			   
+			   get it..
+
 			   We don't want dcc servers to time out. */
 			dcc_close(dcc);
 		}
@@ -479,7 +479,7 @@ static void event_no_such_nick(IRC_SERVER_REC *server, char *data)
 
 		next = tmp->next;
 		if (!dcc_is_connected(dcc) && dcc->server == server &&
-		    dcc->nick != NULL && g_strcasecmp(dcc->nick, nick) == 0)
+		    dcc->nick != NULL && g_ascii_strcasecmp(dcc->nick, nick) == 0)
 			dcc_close(dcc);
 	}
 
@@ -515,7 +515,7 @@ static void cmd_dcc_close(char *data, IRC_SERVER_REC *server)
 		DCC_REC *dcc = tmp->data;
 
 		next = tmp->next;
-		if (dcc->type == type && g_strcasecmp(dcc->nick, nick) == 0 &&
+		if (dcc->type == type && g_ascii_strcasecmp(dcc->nick, nick) == 0 &&
 		    (*arg == '\0' || strcmp(dcc->arg, arg) == 0)) {
 			dcc_reject(dcc, server);
 			found = TRUE;
