@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -24,6 +24,7 @@
 #include "servers-redirect.h"
 #include "levels.h"
 #include "settings.h"
+#include "misc.h"
 
 #include "irc.h"
 #include "irc-channels.h"
@@ -48,9 +49,8 @@ static void remove_client(CLIENT_REC *rec)
 		  "Proxy: Client disconnected from %s", rec->host);
 
 	g_free(rec->proxy_address);
-	net_disconnect(rec->handle);
-	g_source_remove(rec->tag);
-	line_split_free(rec->buffer);
+	net_sendbuffer_destroy(rec->handle, TRUE);
+	g_source_remove(rec->recv_tag);
 	g_free_not_null(rec->nick);
 	g_free_not_null(rec->host);
 	g_free(rec);
@@ -153,8 +153,8 @@ static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args,
 
 		params = event_get_params(args, 2, &origin, &target);
 		if (*target == '\0' ||
-		    g_strcasecmp(target, client->proxy_address) == 0 ||
-		    g_strcasecmp(target, client->nick) == 0) {
+		    g_ascii_strcasecmp(target, client->proxy_address) == 0 ||
+		    g_ascii_strcasecmp(target, client->nick) == 0) {
 			proxy_outdata(client, ":%s PONG %s :%s\n",
 				      client->proxy_address,
                                       client->proxy_address, origin);
@@ -165,27 +165,30 @@ static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args,
 	}
 
 	if (strcmp(cmd, "PROXY") == 0) {
-		if (g_strcasecmp(args, "CTCP ON") == 0) {
+		if (g_ascii_strcasecmp(args, "CTCP ON") == 0) {
                         /* client wants all ctcps */
 			client->want_ctcp = 1;
 	                for (tmp = proxy_clients; tmp != NULL; tmp = tmp->next) {
 				CLIENT_REC *rec = tmp->data;
-				if ((g_strcasecmp(client->server->connrec->chatnet,rec->server->connrec->chatnet) == 0) &&
-					(client->tag != rec->tag)) {
+				if ((g_ascii_strcasecmp(client->listen->ircnet,rec->listen->ircnet) == 0) &&
+					/* kludgy way to check if the clients aren't the same */
+					(client->recv_tag != rec->recv_tag)) {
 						if (rec->want_ctcp == 1)
 							proxy_outdata(rec, ":%s NOTICE %s :Another client is now receiving CTCPs sent to %s\n",
-			                                      rec->proxy_address, rec->nick,rec->server->connrec->chatnet);
-						rec->want_ctcp=0;
+			                                      rec->proxy_address, rec->nick, rec->listen->ircnet);
+						rec->want_ctcp = 0;
 		                }
-						                                      
+
 			}
 			proxy_outdata(client, ":%s NOTICE %s :You're now receiving CTCPs sent to %s\n",
-				      client->proxy_address, client->nick,client->server->connrec->chatnet);
-		} else if (g_strcasecmp(args, "CTCP OFF") == 0) {
+				      client->proxy_address, client->nick,client->listen->ircnet);
+		} else if (g_ascii_strcasecmp(args, "CTCP OFF") == 0) {
                         /* client wants proxy to handle all ctcps */
 			client->want_ctcp = 0;
 			proxy_outdata(client, ":%s NOTICE %s :Proxy is now handling itself CTCPs sent to %s\n",
-				      client->proxy_address, client->nick,client->server->connrec->chatnet);
+				      client->proxy_address, client->nick, client->listen->ircnet);
+		} else {
+			signal_emit("proxy client command", 3, client, args, data);
 		}
 		return;
 	}
@@ -199,6 +202,8 @@ static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args,
         /* check if the command could be redirected */
 	if (strcmp(cmd, "WHO") == 0)
 		grab_who(client, args);
+	else if (strcmp(cmd, "WHOWAS") == 0)
+		proxy_redirect_event(client, "whowas", 1, args, -1);
 	else if (strcmp(cmd, "WHOIS") == 0) {
 		char *p;
 
@@ -257,7 +262,8 @@ static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args,
 
 		ignore_next = TRUE;
 		if (*msg != '\001' || msg[strlen(msg)-1] != '\001') {
-			signal_emit("message own_public", 4,
+	        	signal_emit(ischannel(*target) ?
+				    "message own_public" : "message own_private", 4,
 				    client->server, msg, target, target);
 		} else if (strncmp(msg+1, "ACTION ", 7) == 0) {
 			/* action */
@@ -279,6 +285,12 @@ static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args,
 		g_free(params);
 	} else if (strcmp(cmd, "PING") == 0) {
 		proxy_redirect_event(client, "ping", 1, NULL, TRUE);
+	} else if (strcmp(cmd, "AWAY") == 0) {
+		/* set the away reason */
+		if (args != NULL) {
+			g_free(client->server->away_reason);
+			client->server->away_reason = g_strdup(args);
+		}
 	}
 
 	irc_send_cmd(client->server, data);
@@ -286,14 +298,13 @@ static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args,
 
 static void sig_listen_client(CLIENT_REC *client)
 {
-	char tmpbuf[1024], *str, *cmd, *args;
-	int ret, recvlen;
+	char *str, *cmd, *args;
+	int ret;
 
 	g_return_if_fail(client != NULL);
 
 	while (g_slist_find(proxy_clients, client) != NULL) {
-		recvlen = net_receive(client->handle, tmpbuf, sizeof(tmpbuf));
-		ret = line_split(tmpbuf, recvlen, &str, &client->buffer);
+		ret = net_sendbuffer_receive_line(client->handle, &str, 1);
 		if (ret == -1) {
 			/* connection lost */
 			remove_client(client);
@@ -307,7 +318,7 @@ static void sig_listen_client(CLIENT_REC *client)
 		args = strchr(cmd, ' ');
 		if (args != NULL) *args++ = '\0'; else args = "";
 		if (*args == ':') args++;
-		g_strup(cmd);
+		ascii_strup(cmd);
 
 		handle_client_cmd(client, cmd, args, str);
 
@@ -319,6 +330,7 @@ static void sig_listen(LISTEN_REC *listen)
 {
 	CLIENT_REC *rec;
 	IPADDR ip;
+	NET_SENDBUF_REC *sendbuf;
         GIOChannel *handle;
 	char host[MAX_IP_LEN];
 	int port;
@@ -330,10 +342,10 @@ static void sig_listen(LISTEN_REC *listen)
 	if (handle == NULL)
 		return;
 	net_ip2host(&ip, host);
-
+	sendbuf = net_sendbuffer_create(handle, 0);
 	rec = g_new0(CLIENT_REC, 1);
 	rec->listen = listen;
-	rec->handle = handle;
+	rec->handle = sendbuf;
         rec->host = g_strdup(host);
 	if (strcmp(listen->ircnet, "*") == 0) {
 		rec->proxy_address = g_strdup("irc.proxy");
@@ -343,7 +355,7 @@ static void sig_listen(LISTEN_REC *listen)
 		rec->server = servers == NULL ? NULL :
 			IRC_SERVER(server_find_chatnet(listen->ircnet));
 	}
-	rec->tag = g_input_add(handle, G_INPUT_READ,
+	rec->recv_tag = g_input_add(handle, G_INPUT_READ,
 			       (GInputFunction) sig_listen_client, rec);
 
 	proxy_clients = g_slist_prepend(proxy_clients, rec);
@@ -359,7 +371,7 @@ static void sig_incoming(IRC_SERVER_REC *server, const char *line)
 	g_return_if_fail(line != NULL);
 
 	/* send server event to all clients */
-	g_string_sprintf(next_line, "%s\n", line);
+	g_string_printf(next_line, "%s\n", line);
 }
 
 static void sig_server_event(IRC_SERVER_REC *server, const char *line,
@@ -380,9 +392,9 @@ static void sig_server_event(IRC_SERVER_REC *server, const char *line,
 	args = strchr(event+6, ' ');
 	if (args != NULL) *args++ = '\0'; else args = "";
 	while (*args == ' ') args++;
-        g_strdown(event);
+	ascii_strdown(event);
 
-	signal = server_redirect_peek_signal(server, event, args, &redirected);
+	signal = server_redirect_peek_signal(server, nick, event, args, &redirected);
 	if ((signal != NULL && strncmp(signal, "proxy ", 6) != 0) ||
 	    (signal == NULL && redirected)) {
 		/* we want to send this to one client (or proxy itself) only */
@@ -392,11 +404,11 @@ static void sig_server_event(IRC_SERVER_REC *server, const char *line,
 	}
 
 	if (signal != NULL) {
-                server_redirect_get_signal(server, event, args);
+                server_redirect_get_signal(server, nick, event, args);
 		if (sscanf(signal+6, "%p", &client) == 1) {
 			/* send it to specific client only */
 			if (g_slist_find(proxy_clients, client) != NULL)
-				net_transmit(((CLIENT_REC *) client)->handle, next_line->str, next_line->len);
+				net_sendbuffer_send(((CLIENT_REC *) client)->handle, next_line->str, next_line->len);
 			g_free(event);
                         signal_stop();
 			return;
@@ -409,13 +421,12 @@ static void sig_server_event(IRC_SERVER_REC *server, const char *line,
 		/* CTCP - either answer ourself or forward it to one client */
 		for (tmp = proxy_clients; tmp != NULL; tmp = tmp->next) {
 	        	CLIENT_REC *rec = tmp->data;
-		                        
+
 			if (rec->want_ctcp == 1) {
-                        	/* one of the clients answers */ 
-                        	/* patched, so only CTCP for the chatnet where client is connected to will be forwarded */
-                        	if (strstr(rec->proxy_address,server->connrec->chatnet) != NULL) {
-					net_transmit(rec->handle,
-						     next_line->str, next_line->len);
+                        	/* only CTCP for the chatnet where client is connected to will be forwarded */
+                        	if (strstr(rec->proxy_address, server->connrec->chatnet) != NULL) {
+					net_sendbuffer_send(rec->handle,
+							    next_line->str, next_line->len);
 					signal_stop();
 				}
 			}
@@ -453,11 +464,10 @@ static void event_connected(IRC_SERVER_REC *server)
 		if (rec->connected && rec->server == NULL &&
 		    (strcmp(rec->listen->ircnet, "*") == 0 ||
 		     (chatnet != NULL &&
-		      g_strcasecmp(chatnet, rec->listen->ircnet) == 0))) {
+		      g_ascii_strcasecmp(chatnet, rec->listen->ircnet) == 0))) {
 			proxy_outdata(rec, ":%s NOTICE %s :Connected to server\n",
 				      rec->proxy_address, rec->nick);
 			rec->server = server;
-			rec->want_ctcp = 0;
 			proxy_client_reset_nick(rec);
 		}
 	}
@@ -505,7 +515,7 @@ static void event_nick(IRC_SERVER_REC *server, const char *data,
 	if (!IS_IRC_SERVER(server))
 		return;
 
-	if (g_strcasecmp(orignick, server->nick) != 0)
+	if (g_ascii_strcasecmp(orignick, server->nick) != 0)
 		return;
 
 	if (*data == ':') data++;
@@ -529,6 +539,16 @@ static void sig_message_own_public(IRC_SERVER_REC *server, const char *msg,
 		proxy_outserver_all(server, "PRIVMSG %s :%s", target, msg);
 }
 
+static void sig_message_own_private(IRC_SERVER_REC *server, const char *msg,
+                                   const char *target, const char *origtarget)
+{
+	if (!IS_IRC_SERVER(server))
+		return;
+
+	if (!ignore_next)
+		proxy_outserver_all(server, "PRIVMSG %s :%s", target, msg);
+}
+
 static void sig_message_own_action(IRC_SERVER_REC *server, const char *msg,
                                    const char *target)
 {
@@ -547,7 +567,7 @@ static LISTEN_REC *find_listen(const char *ircnet, int port)
 		LISTEN_REC *rec = tmp->data;
 
 		if (rec->port == port &&
-		    g_strcasecmp(rec->ircnet, ircnet) == 0)
+		    g_ascii_strcasecmp(rec->ircnet, ircnet) == 0)
 			return rec;
 	}
 
@@ -646,6 +666,14 @@ static void read_settings(void)
 	}
 }
 
+static void sig_dump(CLIENT_REC *client, const char *data)
+{
+	g_return_if_fail(client != NULL);
+	g_return_if_fail(data != NULL);
+
+	proxy_outdata(client, data);
+}
+
 void proxy_listen_init(void)
 {
 	next_line = g_string_new(NULL);
@@ -658,10 +686,13 @@ void proxy_listen_init(void)
 	signal_add("server event", (SIGNAL_FUNC) sig_server_event);
 	signal_add("event connected", (SIGNAL_FUNC) event_connected);
 	signal_add("server disconnected", (SIGNAL_FUNC) sig_server_disconnected);
-	signal_add("event nick", (SIGNAL_FUNC) event_nick);
+	signal_add_first("event nick", (SIGNAL_FUNC) event_nick);
 	signal_add("message own_public", (SIGNAL_FUNC) sig_message_own_public);
+	signal_add("message own_private", (SIGNAL_FUNC) sig_message_own_private);
 	signal_add("message irc own_action", (SIGNAL_FUNC) sig_message_own_action);
 	signal_add("setup changed", (SIGNAL_FUNC) read_settings);
+
+	signal_add("proxy client dump", (SIGNAL_FUNC) sig_dump);
 }
 
 void proxy_listen_deinit(void)
@@ -676,6 +707,9 @@ void proxy_listen_deinit(void)
 	signal_remove("server disconnected", (SIGNAL_FUNC) sig_server_disconnected);
 	signal_remove("event nick", (SIGNAL_FUNC) event_nick);
 	signal_remove("message own_public", (SIGNAL_FUNC) sig_message_own_public);
+	signal_remove("message own_private", (SIGNAL_FUNC) sig_message_own_private);
 	signal_remove("message irc own_action", (SIGNAL_FUNC) sig_message_own_action);
 	signal_remove("setup changed", (SIGNAL_FUNC) read_settings);
+
+	signal_remove("proxy client dump", (SIGNAL_FUNC) sig_dump);
 }
